@@ -216,7 +216,7 @@ public:
     M17NCandidateList(M17NEngine *engine, InputContext *ic)
         : engine_(engine), ic_(ic) {
         auto *state = ic_->propertyFor(engine_->factory());
-        auto pageSize = GetPageSize(state->mim_->language, state->mim_->name);
+        auto pageSize = GetPageSize(state->mim()->language, state->mim()->name);
 
         const static KeyList selectionKeys{
             Key(FcitxKey_1), Key(FcitxKey_2), Key(FcitxKey_3), Key(FcitxKey_4),
@@ -224,7 +224,7 @@ public:
             Key(FcitxKey_9), Key(FcitxKey_0)};
         setPageSize(pageSize);
         setSelectionKey(selectionKeys);
-        MPlist *head = state->mic_->candidate_list;
+        MPlist *head = state->mic()->candidate_list;
         int index = 0;
         for (; head && mplist_key(head) != Mnil; head = mplist_next(head)) {
             MSymbol key = mplist_key(head);
@@ -253,10 +253,10 @@ public:
                 FCITX_M17N_DEBUG() << "Invalid MSymbol: " << msymbol_name(key);
             }
         }
-        if (state->mic_->candidate_index >= 0 &&
-            state->mic_->candidate_index < totalSize()) {
-            setGlobalCursorIndex(state->mic_->candidate_index);
-            setPage(state->mic_->candidate_index / pageSize);
+        if (state->mic()->candidate_index >= 0 &&
+            state->mic()->candidate_index < totalSize()) {
+            setGlobalCursorIndex(state->mic()->candidate_index);
+            setPage(state->mic()->candidate_index / pageSize);
         }
     }
 
@@ -370,7 +370,7 @@ std::vector<InputMethodEntry> M17NEngine::listInputMethods() {
         m17n_object_unref(info);
 
         InputMethodEntry entry(uniqueName, fxName,
-                               (strcmp(lang, "t") == 0 ? "*" : lang), "m17n");
+                               (strcmp(lang, "t") == 0 ? "mul" : lang), "m17n");
         entry.setConfigurable(true).setIcon(iconName);
         entry.setUserData(std::make_unique<M17NData>(mlang, mname));
         entries.emplace_back(std::move(entry));
@@ -477,6 +477,9 @@ void M17NState::keyEvent(const InputMethodEntry &entry, KeyEvent &keyEvent) {
                    reinterpret_cast<void *>(&M17NState::callback));
         mplist_put(mim_->driver.callback_list, Minput_delete_surrounding_text,
                    reinterpret_cast<void *>(&M17NState::callback));
+    }
+
+    if (!mic_ && mim_) {
         mic_.reset(minput_create_ic(mim_.get(), this));
     }
 
@@ -495,12 +498,19 @@ bool M17NState::keyEvent(const Key &key) {
         FCITX_M17N_DEBUG() << key << " not my dish";
         return false;
     }
+    return handleKey(msym);
+}
+
+bool M17NState::handleKey(MSymbol key) {
+    if (!mic_) {
+        return false;
+    }
     int thru = 0;
-    if (!minput_filter(mic_.get(), msym, nullptr)) {
+    if (!minput_filter(mic_.get(), key, nullptr)) {
         MText *produced = mtext();
         // If input symbol was let through by m17n, let Fcitx handle it.
         // m17n may still produce some text to commit, though.
-        thru = minput_lookup(mic_.get(), msym, NULL, produced);
+        thru = minput_lookup(mic_.get(), key, NULL, produced);
         if (mtext_len(produced) > 0) {
             auto buf = MTextToUTF8(produced);
             ic_->commitString(buf);
@@ -515,29 +525,31 @@ bool M17NState::keyEvent(const Key &key) {
 
 void M17NState::updateUI() {
     ic_->inputPanel().reset();
-    if (mic_->preedit) {
-        auto preedit = MTextToUTF8(mic_->preedit);
-        if (!preedit.empty()) {
-            SetPreedit(ic_, preedit, mic_->cursor_pos);
+    if (mic_) {
+        if (mic_->preedit) {
+            auto preedit = MTextToUTF8(mic_->preedit);
+            if (!preedit.empty()) {
+                SetPreedit(ic_, preedit, mic_->cursor_pos);
+            }
+            FCITX_M17N_DEBUG() << "IM preedit changed to " << preedit;
+        }
+
+        if (mic_->status) {
+            auto mstatus = MTextToUTF8(mic_->status);
+            // toShow = toShow || (strlen(mstatus) != 0);
+            if (!mstatus.empty()) {
+                FCITX_M17N_DEBUG() << "IM status changed to " << mstatus;
+            }
+        }
+        if (mic_->candidate_list && mic_->candidate_show) {
+            auto candList = std::make_unique<M17NCandidateList>(engine_, ic_);
+            if (candList->size()) {
+                candList->setGlobalCursorIndex(mic_->candidate_index);
+                ic_->inputPanel().setCandidateList(std::move(candList));
+            }
         }
-        FCITX_M17N_DEBUG() << "IM preedit changed to " << preedit;
     }
     ic_->updatePreedit();
-
-    if (mic_->status) {
-        auto mstatus = MTextToUTF8(mic_->status);
-        // toShow = toShow || (strlen(mstatus) != 0);
-        if (!mstatus.empty()) {
-            FCITX_M17N_DEBUG() << "IM status changed to " << mstatus;
-        }
-    }
-    if (mic_->candidate_list && mic_->candidate_show) {
-        auto candList = std::make_unique<M17NCandidateList>(engine_, ic_);
-        if (candList->size()) {
-            candList->setGlobalCursorIndex(mic_->candidate_index);
-            ic_->inputPanel().setCandidateList(std::move(candList));
-        }
-    }
     ic_->updateUserInterface(UserInterfaceComponent::InputPanel);
 }
 
@@ -545,15 +557,17 @@ void M17NState::reset() {
     if (!mic_) {
         return;
     }
-    minput_reset_ic(mic_.get());
+    mic_.reset();
     updateUI();
 }
 
-void M17NState::commitPreedit() const {
+void M17NState::commitPreedit() {
     if (!mic_) {
         return;
     }
 
+    // Per minput_reset_ic comment, sending Mnil should commit the preedit.
+    handleKey(Mnil);
     if (!mic_->preedit) {
         return;
     }
